/**
 * @file libsysdiag.h
 * @brief Routines to parse sysdiag.conf for diagnostic/flash update utilities
 *
 * Copyright (C) 2004 IBM Corporation
 *
 * @author Michael Strosaker <strosake@us.ibm.com>
 */

#ifndef _LIBSYSDIAG_H
#define _LIBSYSDIAG_H

#include <stdint.h>

#define PLATFORM_EVENTS_FACILITY	"ppc64 Platform Events"
#define EVENT_TYPE_RTAS_REPORTED	0x00
#define EVENT_TYPE_REPAIR_ACTION	0x80

#define FLAGS_SCANLOG_DUMP		0x01
#define FLAGS_SERVICEABLE_EVENT		0x02

#define SYSDIAG_CONFIG_FILENAME		"/etc/sysdiag.conf"

#define LOCATION_CODE_BUFSIZE		81

#define SYSDIAG_DEVTYPE_VPD		0x0001
#define SYSDIAG_DEVTYPE_OS		0x0002
#define SYSDIAG_DEVTYPE_FW		0x0004
#define SYSDIAG_DEVTYPE_BRIDGE		0x0008
#define SYSDIAG_DEVTYPE_CONTROLLER	0x0010
#define SYSDIAG_DEVTYPE_CHANNEL		0x0020
#define SYSDIAG_DEVTYPE_VIRTUAL		0x0040

#define SYSDIAG_DEVPROP_HOTPLUG		0x0001
#define SYSDIAG_DEVPROP_DLPAR		0x0002
#define SYSDIAG_DEVPROP_INDICATOR_ID	0x0004
#define SYSDIAG_DEVPROP_INDICATOR_INFO	0x0008
#define SYSDIAG_DEVPROP_INDICATOR_FAULT	0x0010
#define SYSDIAG_DEVPROP_VISITED		0x8000

struct sysdiag_device {
	uint16_t type;
	uint16_t properties;
	char loccode_physical[LOCATION_CODE_BUFSIZE];
	char loccode_logical[LOCATION_CODE_BUFSIZE];
	char devname[32];	/* e.g. eth0, sda, sys0, etc */
	char devname_alt[32];	/* either, host0, etc. or empty */
	char devname_generic[32];	/* either sg0, sg1, etc. or empty */
	char osname[64];	/* e.g. 0000:c0:01.0 */
	char osclass[16];	/* e.g. net, scsi_device, scsi_host, etc. */
	char osclass_alt[16];	/* either block or empty */
	char osclass_generic[16];	/* either scsi_generic or empty */
	char devspec[128];	/* e.g. /pci@800000020000002/pci@2,2 */
	char description[256];
	char bridge[32];	/* e.g. 0000:00:02.2 */
	char hbtl[32];		/* host/bus/target/lun, e.g. 0:0:3:0 */
	char driver[32];
	char microcode[128];
	char vpd_stanza[1024];
	struct sysdiag_device *container;
	struct sysdiag_device *contains;
	struct sysdiag_device *next_in_container;
	struct sysdiag_device *next;
};

struct sysdiag_diag_app {
	/* Devices to which this application applies */
	char code[8];
	char compare[3];
	char value[100];

	/* Contents of the TEST part of the stanza */
	char name[100];
	int test_lvl;
	int report_lvl;
	int disruptive;
	int interactive;
	int exclusive;
	int test_order;
	char reqd_rpms[256];
	char reqd_files[256];
	char script[1024];
	struct sysdiag_diag_app *next;
	struct sysdiag_diag_app *prev;
};

struct sysdiag_flash_app {
	/* Devices to which this application applies */
	char code[8];
	char compare[3];
	char value[100];

	/* Contents of the FLASH_UPDATE part of the stanza */
	char name[100];
	int disruptive;
	char reqd_rpms[256];
	char reqd_files[256];
	char script[1024];
	struct sysdiag_flash_app *next;
	struct sysdiag_flash_app *prev;
};

/* Service routines */
int sysdiag_display_src(const char *src, const char *location_code,
		int sys_attn, char *buf, size_t size);
int sysdiag_system_attention_off(const char *loc_code);

/* Inventory routines */
struct sysdiag_device *sysdiag_get_devices();
void sysdiag_free_devlist(struct sysdiag_device *);
void sysdiag_devlist_mark_unvisited(struct sysdiag_device *);
int sysdiag_get_vpd_from_stanza(char *stanza, const char *vpd, char *buffer,
		size_t);

/* Routines for retrieving lists of diagnostic applications */
struct sysdiag_diag_app *sysdiag_get_all_diagnostic_applications(char *errbuf,
	size_t errbuf_size);
struct sysdiag_diag_app *sysdiag_get_diagnostic_applications(
	struct sysdiag_diag_app *diags, struct sysdiag_device *dev);
void sysdiag_free_diag_app_list(struct sysdiag_diag_app *);

/* Routines for retrieving lists of microcode update applications */
struct sysdiag_flash_app *sysdiag_get_all_flash_update_applications(
	char *errbuf, size_t errbuf_size);
struct sysdiag_flash_app *sysdiag_get_flash_update_applications(
	struct sysdiag_flash_app *apps, struct sysdiag_device *dev,
	const char *file0, const char *file1);
void sysdiag_free_flash_app_list(struct sysdiag_flash_app *);

#endif
